@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.server;
@@ -9,26 +9,24 @@ import java.io.IOException;
 import java.net.ServerSocket;
 import java.net.Socket;
 import java.net.UnknownHostException;
-import java.sql.Connection;
-import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Map;
-import java.util.Properties;
 import java.util.Set;
-import org.h2.Driver;
+import java.util.concurrent.ConcurrentHashMap;
+
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
+import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
-import org.h2.util.JdbcUtils;
+import org.h2.util.MathUtils;
 import org.h2.util.NetUtils;
 import org.h2.util.StringUtils;
 import org.h2.util.Tool;
+import org.h2.util.Utils10;
 
 /**
  * The TCP server implements the native H2 database server protocol.
@@ -49,8 +47,7 @@ public class TcpServer implements Service {
      */
     private static final String MANAGEMENT_DB_PREFIX = "management_db_";
 
-    private static final Map<Integer, TcpServer> SERVERS =
-            Collections.synchronizedMap(new HashMap<Integer, TcpServer>());
+    private static final ConcurrentHashMap<Integer, TcpServer> SERVERS = new ConcurrentHashMap<>();
 
     private int port;
     private boolean portIsSet;
@@ -64,8 +61,8 @@ public class TcpServer implements Service {
     private String baseDir;
     private boolean allowOthers;
     private boolean isDaemon;
-    private boolean ifExists;
-    private Connection managementDb;
+    private boolean ifExists = true;
+    private JdbcConnection managementDb;
     private PreparedStatement managementDbAdd;
     private PreparedStatement managementDbRemove;
     private String managementPassword = "";
@@ -85,22 +82,21 @@ public class TcpServer implements Service {
     }
 
     private void initManagementDb() throws SQLException {
-        Properties prop = new Properties();
-        prop.setProperty("user", "");
-        prop.setProperty("password", managementPassword);
+        if (managementPassword.isEmpty()) {
+            managementPassword = StringUtils.convertBytesToHex(MathUtils.secureRandomBytes(32));
+        }
         // avoid using the driver manager
-        Connection conn = Driver.load().connect("jdbc:h2:" +
-                getManagementDbName(port), prop);
+        JdbcConnection conn = new JdbcConnection("jdbc:h2:" + getManagementDbName(port), null, "", managementPassword,
+                false);
         managementDb = conn;
 
         try (Statement stat = conn.createStatement()) {
-            stat.execute("CREATE ALIAS IF NOT EXISTS STOP_SERVER FOR \"" +
-                    TcpServer.class.getName() + ".stopServer\"");
+            stat.execute("CREATE ALIAS IF NOT EXISTS STOP_SERVER FOR '" + TcpServer.class.getName() + ".stopServer'");
             stat.execute("CREATE TABLE IF NOT EXISTS SESSIONS" +
-                    "(ID INT PRIMARY KEY, URL VARCHAR, USER VARCHAR, " +
-                    "CONNECTED TIMESTAMP)");
+                    "(ID INT PRIMARY KEY, URL VARCHAR, `USER` VARCHAR, " +
+                    "CONNECTED TIMESTAMP(9) WITH TIME ZONE)");
             managementDbAdd = conn.prepareStatement(
-                    "INSERT INTO SESSIONS VALUES(?, ?, ?, NOW())");
+                    "INSERT INTO SESSIONS VALUES(?, ?, ?, CURRENT_TIMESTAMP(9))");
             managementDbRemove = conn.prepareStatement(
                     "DELETE FROM SESSIONS WHERE ID=?");
         }
@@ -188,9 +184,10 @@ public class TcpServer implements Service {
                 isDaemon = true;
             } else if (Tool.isOption(a, "-ifExists")) {
                 ifExists = true;
+            } else if (Tool.isOption(a, "-ifNotExists")) {
+                ifExists = false;
             }
         }
-        org.h2.Driver.load();
     }
 
     @Override
@@ -203,6 +200,16 @@ public class TcpServer implements Service {
         return port;
     }
 
+    /**
+     * Returns whether a secure protocol is used.
+     *
+     * @return {@code true} if SSL socket is used, {@code false} if plain socket
+     *         is used
+     */
+    public boolean getSSL() {
+        return ssl;
+    }
+
     /**
      * Check if this socket may connect to this server. Remote connections are
      * not allowed if the flag allowOthers is set.
@@ -245,9 +252,11 @@ public class TcpServer implements Service {
         try {
             while (!stop) {
                 Socket s = serverSocket.accept();
-                TcpServerThread c = new TcpServerThread(s, this, nextThreadId++);
+                Utils10.setTcpQuickack(s, true);
+                int id = nextThreadId++;
+                TcpServerThread c = new TcpServerThread(s, this, id);
                 running.add(c);
-                Thread thread = new Thread(c, threadName + " thread");
+                Thread thread = new Thread(c, threadName + " thread-" + id);
                 thread.setDaemon(isDaemon);
                 c.setThread(thread);
                 thread.start();
@@ -423,6 +432,7 @@ public class TcpServer implements Service {
      * @param force if the server should be stopped immediately
      * @param all whether all TCP servers that are running in the JVM should be
      *            stopped
+     * @throws SQLException on failure
      */
     public static synchronized void shutdown(String url, String password,
             boolean force, boolean all) throws SQLException {
@@ -436,17 +446,9 @@ public class TcpServer implements Service {
                 }
             }
             String db = getManagementDbName(port);
-            try {
-                org.h2.Driver.load();
-            } catch (Throwable e) {
-                throw DbException.convert(e);
-            }
             for (int i = 0; i < 2; i++) {
-                Connection conn = null;
-                PreparedStatement prep = null;
-                try {
-                    conn = DriverManager.getConnection("jdbc:h2:" + url + "/" + db, "", password);
-                    prep = conn.prepareStatement("CALL STOP_SERVER(?, ?, ?)");
+                try (JdbcConnection conn = new JdbcConnection("jdbc:h2:" + url + '/' + db, null, "", password, true)) {
+                    PreparedStatement prep = conn.prepareStatement("CALL STOP_SERVER(?, ?, ?)");
                     prep.setInt(1, all ? 0 : port);
                     prep.setString(2, password);
                     prep.setInt(3, force ? SHUTDOWN_FORCE : SHUTDOWN_NORMAL);
@@ -466,9 +468,6 @@ public class TcpServer implements Service {
                     if (i == 1) {
                         throw e;
                     }
-                } finally {
-                    JdbcUtils.closeSilently(prep);
-                    JdbcUtils.closeSilently(conn);
                 }
             }
         } catch (Exception e) {
