@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.util;
@@ -16,7 +16,7 @@ import java.util.concurrent.ConcurrentHashMap;
  */
 public class DebuggingThreadLocal<T> {
 
-    private final ConcurrentHashMap<Long, T> map = new ConcurrentHashMap<Long, T>();
+    private final ConcurrentHashMap<Long, T> map = new ConcurrentHashMap<>();
 
     public void set(T value) {
         map.put(Thread.currentThread().getId(), value);
@@ -39,7 +39,7 @@ public class DebuggingThreadLocal<T> {
      * @return a HashMap containing a mapping from thread-id to value
      */
     public HashMap<Long, T> getSnapshotOfAllThreads() {
-        return new HashMap<Long, T>(map);
+        return new HashMap<>(map);
     }
 
 }
