@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  */
 /*
  * Licensed to the Apache Software Foundation (ASF) under one or more
@@ -23,7 +23,6 @@ package org.h2.test.trace;
 import java.math.BigDecimal;
 import java.util.ArrayList;
 
-import org.h2.util.New;
 import org.h2.util.StringUtils;
 
 /**
@@ -189,7 +188,7 @@ class Parser {
                     number.indexOf('.') >= 0) {
                 Double v = Double.parseDouble(number);
                 return new Arg(double.class, v);
-            } else if (number.endsWith("L") || number.endsWith("l")) {
+            } else if (number.endsWith("l")) {
                 Long v = Long.parseLong(number.substring(0, number.length() - 1));
                 return new Arg(long.class, v);
             } else {
@@ -209,7 +208,7 @@ class Parser {
                     read("[");
                     read("]");
                     read("{");
-                    ArrayList<Object> values = New.arrayList();
+                    ArrayList<Object> values = new ArrayList<>();
                     do {
                         values.add(parseValue().getValue());
                     } while (readIf(","));
@@ -250,7 +249,7 @@ class Parser {
 
     private void parseCall(String objectName, Object o, String methodName) {
         stat.setMethodCall(objectName, o, methodName);
-        ArrayList<Arg> args = New.arrayList();
+        ArrayList<Arg> args = new ArrayList<>();
         read("(");
         while (true) {
             if (readIf(")")) {
